package physbeans.editors;

import java.util.*;
import java.awt.*;
import java.beans.*;

/**
 * Property editor for float arrays
 */
public class ArrayOfFloatEditor extends PropertyEditorSupport {
  
  /**
   * Returns a string that represents the current float array.
   */
  public String getAsText() {
    if (getValue() == null)
      return "null";
    
    float[] v = (float[]) getValue();
    int dim = v.length;
    
    String result = "[";
    for (int i=0; i<dim-1; i++) {
      result += v[i] + ",";
    }
    result += v[dim-1] + "]";
    
    return result;
  }
  
  /**
   * Reads the value of the property by parsing a string.
   */
  public void setAsText(String text) throws IllegalArgumentException {
    if (text.equals("null") || text.equals("")) {
      setValue(null);
      return;
    }
    
    float[] v = null;
    StringTokenizer toc = new StringTokenizer(text, "[] ,;");
    int dim = toc.countTokens();
    v = new float[dim];
    
    try {
      for (int i=0; i<dim; i++) {
        v[i] = Float.parseFloat(toc.nextToken());
      }
      
    } catch (NumberFormatException e) {
      wrongFormat(text);
    }
    
    setValue(v);
  }
  
  /**
   * Throws an exception if the text has a wrong format.
   */
  protected void wrongFormat(String text) throws IllegalArgumentException {
    throw new IllegalArgumentException("wrong format in " + text);
  }
  
  /**
   * Returns a string with java code that initializes a DVector.
   */
  public String getJavaInitializationString() {
    float[] v = (float[]) getValue();
    String result = "new float[]{";
    int dim = v.length;
    
    for (int i=0; i<dim - 1; i++) {
      result +=  v[i] + "f,";
    }
    result += v[dim-1] + "f}";
    
    return result;
  }
  
  /**
   * Returns true if a custom editor exists.
   */
  public boolean supportsCustomEditor() {
    return true;
  }
  
  /**
   * Returns an instance of a new custom editor.
   */
  public Component getCustomEditor() {
    return new ArrayOfFloatCustomEditor(this);
  }
  
}
