package physbeans.func;

import java.io.*;
import physbeans.math.*;
import static java.lang.Math.*;

/**
 * computes cartesian components from length and angle of a 2d vector
 */
public class Polar2CartFunction extends GenericNdFunction implements Serializable {
  
  /**
   * constructs a standard function with 2 inputs and 2 outputs
   */
  public Polar2CartFunction() {
    super(2, 2);
  }
  
  /**
   * Computes cartesian components from length and angle.
   */
  protected DVector computeFunction(DVector in) {
    DVector out = new DVector(2);
    
    double r = in.get(0);
    double phi = in.get(1);
    
    out.set(0, r*cos(phi));
    out.set(1, r*sin(phi));
    
    return out;
  }
  
  // conveniance functions to simplify event routing
  public double getX() {
    return getOutputValues(0);
  }
  public double getY() {
    return getOutputValues(1);
  }
  public void setR(double val) {
    setInputValues(0, val);
  }
  public void setPhi(double val) {
    setInputValues(1, val);
  }
}
