package physbeans.inout;

import java.awt.*;
import java.awt.event.*;
import java.net.*;
import javax.swing.*;
import physbeans.event.*;
import physbeans.math.*;

/**
 * Small panel containing two buttons '+' and 'minus'and descriptions
 */
public class ScalePanel extends JPanel {

  // GUI elements
  protected JLabel scaleTitle;
  protected JButton plusButton; // doubles scale
  protected JLabel scaleLabel; // shows scale
  protected JButton minusButton; // halves scale

  // strings showing the scale
  // array has to be symmetric around 1 !!
  protected String[] scaleTexts = {"1/16", "1/8", "1/4", "1/2", "1", "2", "4", "8", "16"};
  protected int scaleIndex; // index into scaleTexts
  protected DVector lastScaleChange;   // for simple get function

  public ScalePanel() {
    super();
    setLayout(new FlowLayout(FlowLayout.CENTER, 2, 3));
    lastScaleChange = new DVector(1.0);
    buildGUI();
  }

  protected void buildGUI() {
    scaleTitle = new JLabel("Scale: ", JLabel.CENTER);
    scaleTitle.setFont(null);

    // get icons for +/- buttons
    ClassLoader myCL = getClass().getClassLoader();
    ImageIcon minusIcon = null;
    URL minusURL = myCL.getResource("physbeans/resources/minus.gif");
    if (minusURL != null) {
      minusIcon = new ImageIcon(minusURL);
    }
    ImageIcon plusIcon = null;
    URL plusURL = myCL.getResource("physbeans/resources/plus.gif");
    if (plusURL != null) {
      plusIcon = new ImageIcon(plusURL);
    }

    plusButton = new JButton(plusIcon);
    plusButton.setOpaque(false);
    plusButton.setMargin(new Insets(1, 1, 1, 1));
    plusButton.addActionListener(new PlusButtonListener());

    scaleIndex = scaleTexts.length / 2;    // middle value, == "1"
    scaleLabel = new JLabel(" " + scaleTexts[scaleIndex] + " ", JLabel.CENTER);
    scaleLabel.setFont(null);
    // scaleLabel should not change size
    // size it with standard font
    FontMetrics fm = getFontMetrics(new Font("Dialog", Font.PLAIN, 11));
    int justw = fm.stringWidth("1/16"); // get size of justString using given font
    int justh = fm.getHeight();
    scaleLabel.setPreferredSize(new Dimension(justw + 4, justh));

    minusButton = new JButton(minusIcon);
    minusButton.setOpaque(false);
    minusButton.setMargin(new Insets(1, 1, 1, 1));
    minusButton.addActionListener(new MinusButtonListener());

    setOpaque(false);
    setFont(null);
    add(scaleTitle);
    add(plusButton);
    add(scaleLabel);
    add(minusButton);
  }

  // listeners
  protected class PlusButtonListener implements ActionListener {

    public void actionPerformed(ActionEvent e) {
      increaseScale();
    }
  }

  protected class MinusButtonListener implements ActionListener {

    public void actionPerformed(ActionEvent e) {
      decreaseScale();
    }
  }

  /**
   * divide scale by a factor of two
   */
  protected void decreaseScale() {
    // do nothing if minimal value reached
    if (scaleIndex == 0) {
      return;
    }

    // change label
    scaleIndex--;
    scaleLabel.setText(scaleTexts[scaleIndex]);
    changeScale(2.0);
  }

  /**
   * enlarge scale by a factor of two
   */
  protected void increaseScale() {
    // do nothing if maximal value reached
    if (scaleIndex == scaleTexts.length - 1) {
      return;
    }

    // ok it is not maximal
    scaleIndex++;
    scaleLabel.setText(scaleTexts[scaleIndex]);
    changeScale(0.5);
  }

  /**
   * Changes scale of the screen.
   */
  protected void changeScale(double d) {
    lastScaleChange = new DVector(d);
    fireVectorEvent(new VectorEvent(this, lastScaleChange));
  }

  /**
   * Sets label of scale controls.
   */
  public void setScaleLabel(String s) {
    scaleTitle.setText(s);
  }

  /**
   * Gets label of scale controls.
   */
  public String getScaleLabel() {
    return scaleTitle.getText();
  }

  /**
   * Sets initial value of the scale index.
   */
  public void setInitialScaleIndex(int newIndex) {
    while (scaleIndex < newIndex) {
      increaseScale();
    }

    while (scaleIndex > newIndex) {
      decreaseScale();
    }
  }

  /**
   * Gets initial value of the scale index.
   */
  public int getInitialScaleIndex() {
    return scaleIndex;
  }

  /**
   * Gets last scale change. Easy access function for listeners.
   */
  public DVector getScaleChange() {
    return lastScaleChange;
  }

  // the usual support for VectorChange events
  protected transient VectorSupport vecSup = new VectorSupport();

  /**
   * register vector listener
   */
  public synchronized void addVectorListener(VectorListener listener) {
    vecSup.addVectorListener(listener);
  }

  /**
   * de-register vector listener
   */
  public synchronized void removeVectorListener(VectorListener listener) {
    vecSup.removeVectorListener(listener);
  }

  /**
   * fires vector event
   */
  public void fireVectorEvent(VectorEvent e) {
    vecSup.fireVectorEvent(e);
  }
}
