package physbeans.math;

import java.io.*;
import java.util.*;

/**
 * Vector of integers.
 */
public class IVector implements Serializable {

  protected int[] v;
  static final long serialVersionUID = 2191608094131739176L;

  /**
   * Creates a 2-dim vector from two given int values.
   */
  public IVector(int v1, int v2) {
    this.v = new int[]{v1, v2};
  }

  /**
   * Creates a 3-dim vector from three given int values.
   */
  public IVector(int v1, int v2, int v3) {
    this.v = new int[]{v1, v2, v3};
  }

  /**
   * Creates a 4-dim vector from four given int values.
   */
  public IVector(int v1, int v2, int v3, int v4) {
    this.v = new int[]{v1, v2, v3, v4};
  }

  /**
   * Construct an n-dim vector of zeros
   * @param n    dimension of vector.
   */
  public IVector(int n) {
    v = new int[n];
  }

  /**
   * Construct a vector from a 1-D array without copying
   * @param v     array of doubles.
   */
  public IVector(int[] v) {
    this.v = v;
  }

  /**
   * Gets a single element.
   */
  public int get(int i) {
    return v[i];
  }

  /**
   * Gets the dimension of the IVector, i.e. its length.
   */
  public int getDimension() {
    return v.length;
  }

  /**
   * Sets a single element.
   */
  public void set(int i, int s) {
    v[i] = s;
  }

  /**
   * reads a String into a IVector
   */
  static public IVector valueOf(String s) {
    StringTokenizer toc = new StringTokenizer(s, "[] ,;");
    int dim = toc.countTokens();
    IVector v = new IVector(dim);

    for (int i = 0; i < dim; i++) {
      String word = toc.nextToken();
      try {
        v.set(i, Integer.parseInt(word));
      } catch (NumberFormatException e) {
        throw new IllegalArgumentException("wrong format in " + s);
      }
    }

    return v;
  }

  /**
   * Returns largest element of the vector.
   */
  public int max() {
    int maxVal = Integer.MIN_VALUE;
    for (int i = 0; i < v.length; i++) {
      maxVal = Math.max(maxVal, v[i]);
    }
    return maxVal;
  }

  /**
   * Returns smallest element of the vector.
   */
  public int min() {
    int minVal = Integer.MAX_VALUE;
    for (int i = 0; i < v.length; i++) {
      minVal = Math.min(minVal, v[i]);
    }
    return minVal;
  }

  /**
   * returns a String representation of the IVector
   */
  public String toString() {
    int n = v.length;
    if (n == 0) {
      return "[]";
    }

    String result = "[";
    for (int i = 0; i < n - 1; i++) {
      result += v[i] + ",";
    }
    result += v[n - 1] + "]";

    return result;
  }
}
