package physbeans.math;

import physbeans.model.*;

/**
 * A LagrangeInterpolator can be used to interpolate values between
 * a series of 2-dimensional points. The interpolation function is
 * the Langrange interpolation polynomial of a degree equal to the
 * number of points in the series minus one.
 *
 * @author Didier H. Besset
 * adapted to PhysBeans   PJ
 */
public class LagrangeInterpolator extends Interpolator1D {

  
  /**
   * Constructor method.
   * @param pts the series of points.
   */
  public LagrangeInterpolator(Point2dVector pts) {
    super(pts);
  }

  /**
   * Computes the interpolated y value for a given x value.
   * 
   * @param aNumber x value.
   * @return interpolated y value.
   */
  public double evaluate( double aNumber) {
    double norm = 1.0;
    int size = points.size();
    double products[] = new double[size];
    for ( int i = 0; i < size; i++)
      products[i] = 1;
    double dx;
    for ( int i = 0; i < size; i++) {
      dx = aNumber - points.getX(i);
      if ( dx == 0 )
        return points.getY(i);
      norm *= dx;
      for ( int j = 0; j < size; j++) {
        if ( i != j)
          products[j] *= points.getX(j)
          - points.getX(i);
      }
    }
    double answer = 0.0;
    for ( int i = 0; i < size; i++)
      answer += points.getY(i) / (products[i] * (aNumber - points.getX(i)));
    return norm * answer;
  }
}