package physbeans.math;

import static java.lang.Math.*;     // readable math at last!
import static physbeans.math.Maths.*;

/**
 * Twodimensional line usually given by two points.
 */
public class LineSegment2D {
  
  protected DVector p1, p2;
  
  /**
   * Creates a new LineSegment2D from two points.
   */
  public LineSegment2D(DVector p1, DVector p2) {
    this.p1 = p1;
    this.p2 = p2;
  }
  
  /**
   * Computes the ray parameter of the first hit point of the line segment
   * and the ray.
   * Returns NaN, if no hit point exists.
   */
  public double getRayHitParameter(Ray2D ray) {
    DVector p = ray.getStartPoint();
    double sa = sin(ray.getAngle());
    double ca = cos(ray.getAngle());
    
    // get line parameter (must be in (0,1)!)
    double mu = (sa*(p1.x() - p.x()) - ca*(p1.y() - p.y())) /
        (sa*(p1.x() - p2.x()) - ca*(p1.y() - p2.y()));
    
    double l = Double.NaN;
    if ((mu > 0) && (mu < 1)) {
      if (isZero(ca)) {  // careful with vertical lines!
        l = (p1.y() - p.y() - mu*(p1.y() - p2.y()))/sa;
      } else {
        l = (p1.x() - p.x() - mu*(p1.x() - p2.x()))/ca;
      }
    }
    
    if (isStrictlyPositive(l)) {
      return l;
    } else {
      return Double.NaN;
    }
  }
  
  /**
   * Computes the hit point of the line segment and the ray.
   */
  public DVector getRayHitPoint(Ray2D ray) {
    
    double l = getRayHitParameter(ray);
    if (Double.isNaN(l)) {
      return new DVector(Double.NaN, Double.NaN);
    } else {
      return ray.getPointAtParameter(l);
    }
  }
}
