package physbeans.math;

/**
 * solve an ODE by giving the "next" value for x
 * generic version
 */
public abstract class ODESolver {

  // ode to solve
  protected ODE  ode;
  
  // current values, public to speed up access
  public double   t;
  public DVector  x;
  
  /**
   * construct solver for a given ODE
   */
  public ODESolver(ODE ode) {
    this.ode = ode;
    t = ode.getT0();
    x = ode.getX0();
  }

  /**
   * computes x(t+dt)<br>
   * returns number of steps needed
   */
  public abstract int nextStep(double dt) throws StepsizeTooSmallException;

  /**
   * start again with initial values of the ODE
   */
  public void restart() {
    t = ode.getT0();
    x = ode.getX0();
  }
  
  /**
   * gets ode to be solved
   */
  public ODE getOde() {
    return this.ode;
  }
  
  /**
   * sets ode to be solved
   */
  public void setOde(ODE ode) {
    this.ode = ode;
  }
  
  /** 
   * gets current time
   */
  public double getTime() {
    return t;
  }
  
  /** 
   * sets current time
   */
  public void setTime(double time) {
    t = time;
  }
  
  /** 
   * gets (copy of) current state vector
   */
  public DVector getValues() {
    return x.copy();
  }
  
  /** 
   * sets current state vector to (copy of) values
   */
  public void setValues(DVector values) {
    x = values.copy();
  }
  
}
