package physbeans.model;

import physbeans.math.*;

/**
 * DataBuffer, where the first buffer is used for time tags.<br>
 * When new data is stored, its time tag is computed simply by adding a fixed
 * increment (timePerTick) to the last time value.
 */
public class TimedDataBuffer extends DataBuffer {
  
  protected double timePerTick;
  protected double time;
  
  /** Creates a new instance of TimedDataBuffer */
  public TimedDataBuffer() {
    super();
    
    timePerTick = 0.1;
    time = 0.0;
  }
  
  /**
   * Gets the time per tick, i.e. the amount by which the internal time is
   * incremented each time a set of new values arrives.
   */
  public double getTimePerTick() {
    return timePerTick;
  }
  
  /**
   * Sets the time per tick, i.e. the amount by which the internal time is
   * incremented each time a set of new values arrives.
   */
  public void setTimePerTick(double dt) {
    timePerTick = dt;
    trigger();
  }
  
  /**
   * Stores the data in d at the next position.
   */
  public void storeData(DVector v) {
    int newLength = v.getDimension() + 1;
    DVector vNew = new DVector(newLength);
    
    vNew.set(0, time);
    for (int i = 1; i < newLength; i++) {
      vNew.set(i, v.get(i - 1));
    }
    super.storeData(vNew);
    
    time += timePerTick;
  }
  
  /**
   * Restarts the buffer.
   */
  public void restart() {
    pos = 0;
    time = 0.0;
    trigger();
  }
  
}
