package physbeans.phys;

/**
 * A medium with linearly changing wave velocity in a given horizontal layer.
 */
public class GradientWaveMedium extends WaveMedium {
  
  protected double cUp, cLow;
  protected double yLow, yUp;
  
  /**
   * Creates a new instance of GradientWaveMedium 
   */
  public GradientWaveMedium() {
    super();
    cUp = 0.7;
    cLow = 1.0;
    yUp = 0.5;
    yLow = -0.5;
  }
  
  protected void setGrid() {
    if ((field == null) || (grid == null)) {
      return;
    }
    
    initGrid();
    
    // beware: y grid indices are upside down!
    int yu = field.getIndexForY(yLow);
    int yl = field.getIndexForY(yUp);
    double dc = (cUp - cLow)/(yl - yu);
    
    for (int y = 0; y <= yl; y++) {
      for (int x = 0; x < field.getGridSizeX(); x++) {
        grid[y][x] = cUp*cUp;
      }
    }
    
    for (int y = yl + 1; y < yu; y++) {
      for (int x = 0; x < field.getGridSizeX(); x++) {
        double c = cUp + (y - yl)*dc;
        grid[y][x] = c*c;
      }
    }
    
    for (int y = yu; y < field.getGridSizeY(); y++) {
      for (int x = 0; x < field.getGridSizeX(); x++) {
        grid[y][x] = cLow*cLow;
      }
    }
    
    trigger();
  }
  
  /**
   * Gets the position of the upper boundary, where the wave velocity reaches
   * its largest value.
   */
  public double getUpperBoundary() {
    return yUp;
  }
  
  /**
   * Sets the position of the upper boundary, where the wave velocity reaches
   * its largest value.
   */
  public void setUpperBoundary(double p) {
    yUp = p;
    setGrid();
  }
  
  /**
   * Gets the position of the lower boundary, where the wave velocity reaches
   * its smallest value.
   */
  public double getLowerBoundary() {
    return yLow;
  }
  
  /**
   * Sets the position of the lower boundary, where the wave velocity reaches
   * its smallest value.
   */
  public void setLowerBoundary(double p) {
    yLow = p;
    setGrid();
  }
  
  /**
   * Gets the wave velocity on the upper side.
   */
  public double getUpperVelocity() {
    return cUp;
  }
  
  /**
   * Sets the wave velocity on the upper side.
   */
  public void setUpperVelocity(double c) {
    cUp = c;
    setGrid();
  }
  
  /**
   * Gets the wave velocity on the lower side.
   */
  public double getLowerVelocity() {
    return cLow;
  }
  
  /**
   * Sets the wave velocity on the lower side.
   */
  public void setLowerVelocity(double c) {
    cLow = c;
    setGrid();
  }
}
