package physbeans.phys;

/**
 * A simple vertical or horizontal wall.
 */
public class SimpleWall extends WaveWall {
  
  protected boolean horizontal;
  protected double pos;
  
  /** Creates a new instance of SimpleWall */
  public SimpleWall() {
    super();
    horizontal = false;
    pos = 0.0;
  }
  
  protected void setGrid() {
    if ((field == null) || (grid == null)) {
      return;
    }
    
    initGrid();
    
    if (horizontal) {
      int y = field.getIndexForY(pos);
      for (int i = 0; i < field.getGridSizeX(); i++) {
        grid[y][i] = 1.0;
      }
    } else {
      int x = field.getIndexForX(pos);
      for (int i = 0; i < field.getGridSizeY(); i++) {
        grid[i][x] = 1.0;
      }
    }
    
    trigger();
  }
  
  /**
   * Gets the value of the horizontal flag.<br>
   * If true the wall is horizontal, else vertical.
   */
  public boolean isHorizontal() {
    return horizontal;
  }
  
  /**
   * Sets the value of the horizontal flag.<br>
   * If true the wall is horizontal, else vertical.
   */
  public void setHorizontal(boolean h) {
    horizontal = h;
    setGrid();
  }
  
  /**
   * Gets the position of the wall.<br>
   * This is an x coordinate for a vertical wall, a y coordinate for a
   * horizontal one.
   */
  public double getPosition() {
    return pos;
  }
  
  /**
   * Sets the position of the wall.<br>
   * This is an x coordinate for a vertical wall, a y coordinate for a
   * horizontal one.
   */
  public void setPosition(double p) {
    pos = p;
    setGrid();
  }
}
