package physbeans.views;

import java.awt.*;
import java.awt.geom.*;
import java.io.*;
import physbeans.math.*;

/**
 * Displays a mass represented as a rectangle or a sphere.
 */
public class SingleMassView extends RestartableView implements Serializable {
  
  protected double mWidth, mHeight;
  protected boolean square;
  
  /**
   * create a standard square SpringView
   */
  public SingleMassView() {
    super();
    
    mWidth  = 0.4;
    mHeight = 0.4;
    square = true;
  }
  
  /**
   * Draws the mass.
   */
  public void paintLayer(Graphics2D g) {
    
    if (square)  {
      Rectangle2D.Double mass = new Rectangle2D.Double();
      mass.setRect(model.x()-mWidth/2, model.y()-mHeight/2, mWidth, mHeight);
      g.fill(mass);
    } else {
      Arc2D.Double mass = new Arc2D.Double();
      mass.setArc(model.x()-mWidth/2, model.y()-mHeight/2, mWidth, mHeight, 0, 360, 1);
      g.fill(mass);
    }
  }
  
  /**
   * Gets width of the mass.
   */
  public double getMassWidth() {
    return this.mWidth;
  }
  
  /**
   * Sets width of the mass.
   */
  public void setMassWidth(double mWidth) {
    this.mWidth = mWidth;
    update();
  }
  
  /**
   * Gets height of the mass.
   */
  public double getMassHeight() {
    return this.mHeight;
  }
  
  /**
   * Sets height of the mass.
   */
  public void setMassHeight(double mHeight) {
    this.mHeight = mHeight;
    update();
  }
  
  /**
   * Gets position of the mass.
   */
  public DVector getPosition() {
    return getModel();
  }
  
  /**
   * Sets position of the mass.
   */
  public void setPosition(DVector position) {
    setModel(position);
  }
  
  /**
   * Sets x position of the mass.<br>
   * Conveniance function to simplify routing.
   */
  public void setXPosition(double x) {
    model.set(0, x);
    update();
  }
  
  /**
   * Sets y position of the mass.<br>
   * Conveniance function to simplify routing.
   */
  public void setYPosition(double y) {
    model.set(1, y);
    update();
  }
  
  /**
   * Gets square flag.
   */
  public boolean isSquare() {
    return this.square;
  }
  
  /**
   * Sets square flag.<br>
   * If true the mass is represented as a rectangle, else as an oval.
   */
  public void setSquare(boolean square) {
    this.square = square;
    update();
  }
}
