package physbeans.views;

import java.awt.*;
import java.awt.image.*;

/**
 * Color table with a fixed color that changes its transparency from 0 to 1 or
 * from 1 to 0 in inverse mode.
 */
public class TransparentColorTable extends ColorTable {
  
  protected Color color;
  protected boolean inverse;
  
  protected void defineRGBAValues() {
    for (int i=0; i<256; i++) {
      r[i] = (byte) color.getRed();
      g[i] = (byte) color.getGreen();
      b[i] = (byte) color.getBlue();
      a[i] = inverse? (byte) (255-i) : (byte) i;
    }
  }
  
  /**
   * Gets the basic color of the color table.
   */
  public Color getColor() {
    return color;
  }
  
  /**
   * Sets the basic color of the color table.
   */
  public void setColor(Color c) {
    color = c;
    defineRGBAValues();
    createTable();
  }
  
  /**
   * Gets the value of the inverse flag.
   */
  public boolean isInverse() {
    return inverse;
  }
  
  /**
   * Sets the value of the inverse flag.
   */
  public void setInverse(boolean b) {
    inverse = b;
    defineRGBAValues();
    createTable();
  }
  
  protected void initTables() {
    color = Color.BLACK;
    inverse = false;
    super.initTables();
  }
  
}
